﻿using System;
using System.Collections.ObjectModel;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.ServiceModel.Dispatcher;

namespace TestServer
{
    /// <summary>
    /// 글로벌 에러 핸들러 행동 어트리뷰트
    /// </summary>
    public class GlobalErrorHandlerBehaviourAttribute : Attribute, IServiceBehavior
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 에러 핸들러 타입
        /// </summary>
        private readonly Type errorHandlerType;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - GlobalErrorHandlerBehaviourAttribute(errorHandlerType)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="errorHandlerType">에러 핸들러 타입</param>
        public GlobalErrorHandlerBehaviourAttribute(Type errorHandlerType)
        {
            this.errorHandlerType = errorHandlerType;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 무결성 조사하기 - Validate(serviceDescription, serviceHostBase)

        /// <summary>
        /// 무결성 조사하기
        /// </summary>
        /// <param name="serviceDescription">서비스 설명</param>
        /// <param name="serviceHostBase">서비스 호스트 베이스</param>
        public void Validate(ServiceDescription serviceDescription, ServiceHostBase serviceHostBase)
        {
        }

        #endregion
        #region 바인딩 매개 변수 추가하기 - AddBindingParameters(serviceDescription, serviceHostBase, serviceEndpointCollection, bindingParameterCollection)

        /// <summary>
        /// 바인딩 매개 변수 추가하기
        /// </summary>
        /// <param name="serviceDescription">서비스 설명</param>
        /// <param name="serviceHostBase">서비스 호스트 베이스</param>
        /// <param name="serviceEndpointCollection">서비스 종점 컬렉션</param>
        /// <param name="bindingParameterCollection">바인딩 매개 변수 컬렉션</param>
        public void AddBindingParameters(ServiceDescription serviceDescription, ServiceHostBase serviceHostBase, Collection<ServiceEndpoint> serviceEndpointCollection, BindingParameterCollection bindingParameterCollection)
        {
        }

        #endregion
        #region 디스패처 행동 적용하기 - ApplyDispatchBehavior(serviceDescription, serviceHostBase)

        /// <summary>
        /// 디스패처 행동 적용하기
        /// </summary>
        /// <param name="serviceDescription">서비스 설명</param>
        /// <param name="serviceHostBase">서비스 호스트 베이스</param>
        public void ApplyDispatchBehavior(ServiceDescription serviceDescription, ServiceHostBase serviceHostBase)
        {
            IErrorHandler errorHandler = (IErrorHandler)Activator.CreateInstance(this.errorHandlerType);

            foreach(ChannelDispatcherBase channelDispatcherBase in serviceHostBase.ChannelDispatchers)
            {
                ChannelDispatcher channelDispatcher = channelDispatcherBase as ChannelDispatcher;

                if(channelDispatcher != null)
                {
                    channelDispatcher.ErrorHandlers.Add(errorHandler);
                }
            }
        }

        #endregion
    }
}